package com.example.androidtest;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.JavascriptInterface;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import org.json.JSONException;
import org.json.JSONObject;

public class MainActivity extends AppCompatActivity {
    public WebView webview;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        webview = findViewById(R.id.webview);
        WebSettings webSettings = webview.getSettings();
        webSettings.setDomStorageEnabled(true);
        webSettings.setJavaScriptEnabled(true);
        webSettings.setBlockNetworkImage(false);
        // 设置屏幕自适应。
        webSettings.setUseWideViewPort(true);
        webSettings.setLoadWithOverviewMode(true);
        webview.setOverScrollMode(View.OVER_SCROLL_NEVER);
        webSettings.setCacheMode(WebSettings.LOAD_NO_CACHE);
        // 建立JavaScript调用Java接口的桥梁。
        webview.addJavascriptInterface(new testJsInterface(this), "testInterface");
        webview.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                // 返回false表示取消原生的滑动
                return false;
            }
        });
        ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) webview.getLayoutParams();

        params.topToTop = ConstraintLayout.LayoutParams.PARENT_ID;
        params.bottomToBottom = ConstraintLayout.LayoutParams.PARENT_ID;
        params.startToStart = ConstraintLayout.LayoutParams.PARENT_ID;
        params.endToEnd = ConstraintLayout.LayoutParams.PARENT_ID;
        View grayBackground = findViewById(R.id.gray_background);

        grayBackground.setVisibility(View.VISIBLE);

        webview.setLayoutParams(params);

        DisplayMetrics displayMetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        int screenWidth = displayMetrics.widthPixels;

        params.width = (int) (screenWidth * 0.9);
        params.height = ViewGroup.LayoutParams.WRAP_CONTENT;

        WebView.setWebContentsDebuggingEnabled(true);
        webview.loadUrl("file:///android_asset/index.html");
    }

    @Override
    protected void onDestroy() {
        if (webview != null) {
            webview.removeJavascriptInterface("testInterface"); // 移除JavaScript接口
            webview.destroy();
        }
        super.onDestroy();
    }

    public class testJsInterface {
        Activity mActivity;

        /** 实例化接口并传入上下文 */
        testJsInterface(Activity activity) {
            mActivity = activity;
        }
        @JavascriptInterface
        public void getVerifyResult(String jsonString) throws JSONException {
            System.out.println(jsonString);
            try {
                // 将JSON字符串转换为JSONObject
                JSONObject jsonObject = new JSONObject(jsonString);
                String captchaVerifyParam = jsonObject.getString("data");
                String callbackName = jsonObject.getString("callback");

                // 用captchaVerifyParam请求后端接口拿到请求结果
                // JSONObject verifyResult = requestAPI(captchaVerifyParam);

                JSONObject result = new JSONObject();
                // 这里模拟一下返回结果
                result.put("captchaResult", true);
                result.put("bizResult", true);

                String resultString = result.toString(); // result可以转换为一个JSON字符串，注意传给H5之后的数据格式转换
                System.out.println(resultString);

                // 处理完数据后，将结果回传给H5页面
                // 这里需要确保操作在主线程(UI线程)中执行
                webview.post(new Runnable() {
                    @Override
                    public void run() {
                        // 使用evaluateJavascript发送回调到H5页面
                        webview.evaluateJavascript("javascript:" + callbackName + "('" + escapeJavaScriptString(resultString) + "')", null);
                        // 根据result在App测来执行业务自定义提示逻辑，H5中的onBizResultCallback可以置为空函数
                    }
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        @JavascriptInterface
        public void closeWebView() {
            mActivity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mActivity.finish(); // 如果WebView是在单独的Activity中，直接关闭Activity
                    // 如果WebView不在单独的Activity中，可以这样隐藏或移除WebView
                    // webView.setVisibility(View.GONE); // 或 webView.getParent().removeView(webView);
                }
            });
        }
    }

    // 对JavaScript字符串进行转义，避免JavaScript注入问题
    private String escapeJavaScriptString(String string) {
        // 这是一个简单的转义示例。对于更复杂的情况，可能需要更有效的转义方法
        return string.replace("'", "\\'").replace("\n", "\\n").replace("\r", "\\r").replace("\"", "\\\"");
    }
}
